# Generated by Django 4.1.2 on 2022-10-24 05:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drf_operation_log", "0002_auto_20220428_1916"),
    ]

    operations = [
        migrations.AddField(
            model_name="operationlogentry",
            name="extra",
            field=models.JSONField(blank=True, null=True, verbose_name="其他信息"),
        ),
    ]
